@@ -72,7 +72,7 @@ pace <- ancestral.pars
 mpr.help = function (tree, data, cost=NULL) 
 {   
     tree<- reorder(tree, "postorder")     
-    if (class(data) != "phyDat") 
+    if (!inherits(data,"phyDat")) 
         stop("data must be of class phyDat")    
     levels <- attr(data, "levels")
     l = length(levels)
@@ -261,8 +261,6 @@ lowerBound <- function(x, cost=NULL){
 
     if(is.null(cost))cost <- 1 - diag(nc)
 
-
-
     if(any(ust>1)){ 
         ust = ust[ust>1]
         m <- max(ust)    
@@ -287,7 +285,7 @@ upperBound <- function(x, cost=NULL){
 
 
 CI <- function (tree, data, cost=NULL){
-    pscore = sankoff(tree, data, cost=cost)
+    pscore = sankoffNew(tree, data, cost=cost)
     weight = attr(data, "weight")
     data = subset(data, tree$tip.label) 
     m = lowerBound(data, cost=cost)    
@@ -309,7 +307,7 @@ RI <- function (tree, data, cost=NULL)
 
 # not used
 add.one <- function (tree, tip.name, i){
-    if (class(tree) != "phylo") 
+    if (!inherits(tree,"phylo")) 
         stop("tree should be an object of class 'phylo.'")
     nTips = length(tree$tip)
     tmpedge = tree$edge
@@ -429,7 +427,7 @@ old2new.phyDat <- function(obj){
     X = matrix(rep(rowSums(contrast), each=nr),nrow=nr)    
     res <- vector("list", l)
     for(i in 1:l){
-        browser()
+#        browser()
         tmp = X - tcrossprod(obj[[i]], contrast)
         res[[i]] = unlist(apply(tmp, 1, function(x)which(x<1e-6)[1]))
     }
@@ -475,7 +473,7 @@ prepareDataSankoff <- function(data){
 
 sankoff <- function (tree, data, cost = NULL, site = 'pscore') 
 {
-    if (class(data) != "phyDat") 
+    if (!inherits(data,"phyDat")) 
         stop("data must be of class phyDat")
     data <- prepareDataSankoff(data)
     levels <- attr(data, "levels")
@@ -486,8 +484,8 @@ sankoff <- function (tree, data, cost = NULL, site = 'pscore')
         cost <- cost - diag(l)
     }   
     for (i in 1:length(data)) storage.mode(data[[i]]) = "double"
-    if(class(tree)=="phylo") return(fit.sankoff(tree, data, cost, returnData =site))
-    if(class(tree)=="multiPhylo"){
+    if(inherits(tree,"phylo")) return(fit.sankoff(tree, data, cost, returnData =site))
+    if(inherits(tree,"multiPhylo")){
 	    if(is.null(tree$TipLabel))tree = unclass(tree)
 	    return(sapply(tree, fit.sankoff, data, cost, site))
     }    
@@ -574,14 +572,14 @@ indexNNI <- function(tree){
     attr(edgeMatrix, 'root') <-cvector[[min(parent)]]  
     edgeMatrix
 }
-                   
+
         
 sankoff.nni = function (tree, data, cost, ...) 
 {   
     if(is.rooted(tree))tree<- reorder(unroot(tree), "postorder")     
     INDEX <-  indexNNI(tree)
     rootEdges <- attr(INDEX,"root")
-    if (class(data) != "phyDat") 
+    if (!inherits(data,"phyDat")) 
         stop("data must be of class phyDat")
     levels <- attr(data, "levels")
     l = length(levels)
@@ -643,7 +641,7 @@ optim.parsimony <- function(tree,data, method='fitch', cost=NULL, trace=1, rearr
 }
 
 
-pratchet <- function (data, start=NULL, method="fitch", maxit=100, k=5, trace=1, all=FALSE, rearrangements="SPR", ...) 
+pratchet <- function (data, start=NULL, method="fitch", maxit=1000, k=10, trace=1, all=FALSE, rearrangements="SPR", ...) 
 {
     eps = 1e-08
 #    if(method=="fitch" && (is.null(attr(data, "compressed")) || attr(data, "compressed") == FALSE)) 
@@ -688,7 +686,7 @@ pratchet <- function (data, start=NULL, method="fitch", maxit=100, k=5, trace=1,
     for (i in 1:maxit) {
         bstrees <- bootstrap.phyDat(data, FUN, tree = tree, bs = 1, trace = trace, method=method, rearrangements=rearrangements, ...)
         trees <- lapply(bstrees, optim.parsimony, data, trace = trace, method=method, rearrangements=rearrangements, ...)
-        if(class(result)=="phylo")m=1
+        if(inherits(result,"phylo"))m=1
         else m = length(result)
         if(m>0) trees[2 : (1+m)] = result[1:m]
         pscores <- sapply(trees, function(data) attr(data, "pscore"))
@@ -720,7 +718,7 @@ pratchet <- function (data, start=NULL, method="fitch", maxit=100, k=5, trace=1,
 
 
 optim.sankoff <- function(tree, data, cost=NULL, trace=1, ...) {
-    if(class(tree)!="phylo") stop("tree must be of class phylo") 
+    if(!inherits(tree,"phylo")) stop("tree must be of class phylo") 
     if(is.rooted(tree))tree <- unroot(tree)
     if(is.null(attr(tree, "order")) || attr(tree, "order") == "cladewise") tree <- reorder(tree, "postorder")
     if (class(data)[1] != "phyDat") stop("data must be of class phyDat")
@@ -758,11 +756,11 @@ optim.sankoff <- function(tree, data, cost=NULL, trace=1, ...) {
 #
 ptree <- function (tree, data, type = "ACCTRAN", retData = FALSE) 
 {
-    if (class(data) != "phyDat") 
+    if (!inherits(data,"phyDat")) 
         stop("data must be of class phyDat")
     if (is.null(attr(tree, "order")) || attr(tree, "order") == 
         "cladewise") 
-        tree <- reorder(tree, "pruningwise") 
+        tree <- reorder(tree, "pruningwise")  
  #   if (!is.binary.tree(tree)) 
  #       stop("Tree must be binary!")
     tmp = fitch(tree, data, site = "data")
