@@ -26,11 +26,14 @@ phyDat.default <- function (data, levels = NULL, return.index = TRUE, contrast =
     if (is.matrix(data)) 
         nam = row.names(data)
     else nam = names(data)
-    if (class(data) == "DNAbin") 
+    if(is.null(nam))stop("data object must contain taxa names")
+    if (inherits(data,"DNAbin")) 
         data = as.character(data)
     if (is.matrix(data)) 
         data = as.data.frame(t(data), stringsAsFactors = FALSE)
+    if (is.vector(data))data = as.data.frame(t(data), stringsAsFactors = FALSE)
     else data = as.data.frame(data, stringsAsFactors = FALSE)
+    if(length(data[[1]])==1) compress=FALSE 
     if(compress){
         ddd = fast.table(data)
         data = ddd$data
@@ -63,22 +66,18 @@ phyDat.default <- function (data, levels = NULL, return.index = TRUE, contrast =
                 contrast = rbind(contrast, matrix(1, k, l))
         }
     }
-    d = dim(data)
     att = attributes(data) 
-    data = match(unlist(data), all.levels)
-    attr(data, "dim") = d
-    data = as.data.frame(data, stringsAsFactors=FALSE)  
+    data = lapply(data, match, all.levels) # avoid unlist   
     attributes(data) = att
-
+    
     row.names(data) = as.character(1:p)
     data = na.omit(data)
-   
+    
     aaa = match(index, attr(data, "na.action"))
     index = index[is.na(aaa)] 
     index = match(index, unique(index))
     rn = as.numeric(rownames(data))
-    attr(data, "na.action") = NULL  
-        
+    attr(data, "na.action") = NULL        
     weight = weight[rn] 
     p = dim(data)[1]
     names(data) = nam
@@ -102,7 +101,7 @@ phyDat.DNA = function (data, return.index = TRUE)
     if (is.matrix(data)) 
         nam = row.names(data)
     else nam = names(data)
-    if (class(data) == "DNAbin") 
+    if (inherits(data,"DNAbin")) 
         data = as.character(data)
     if (is.matrix(data)) 
         data = as.data.frame(t(data), stringsAsFactors = FALSE)
@@ -163,7 +162,7 @@ phyDat.AA <- function (data, return.index = TRUE)
 {
     if(is.matrix(data)) nam = row.names(data)
     else nam = names(data)  
-    if (class(data) == "DNAbin") 
+    if (inherits(data,"DNAbin")) 
         data = as.character(data)
     if (is.matrix(data)) 
         data = as.data.frame(t(data), stringsAsFactors = FALSE)
@@ -230,7 +229,7 @@ phyDat.codon <- function (data, return.index = TRUE)
 {
     if(is.matrix(data)) nam = row.names(data)
     else nam = names(data)  
-    if (class(data) == "DNAbin") 
+    if (inherits(data,"DNAbin")) 
         data = as.character(data)
 
     if (is.matrix(data)) 
@@ -305,7 +304,7 @@ phyDat.codon <- function (data, return.index = TRUE)
 
 
 as.phyDat <- function (x, ...){
-    if (class(x) == "phyDat") return(x)
+    if (inherits(x,"phyDat")) return(x)
     UseMethod("as.phyDat")
 }
 
@@ -328,7 +327,46 @@ as.phyDat.alignment <- function (x, type="DNA",...)
 }
 
 
-as.alignment.phyDat <- function(x, ...) as.alignment(as.character(x))
+#as.alignment.phyDat <- function(x, ...) as.alignment(as.character(x))
+
+phyDat2alignment <-  function(x){
+    z = as.character(x)
+    nam = rownames(z)
+    type = attr(x, "type")
+    seq <- switch(type, 
+                  DNA = tolower(apply(z, 1, paste, collapse="")), 
+                  AA = toupper(apply(z, 1, paste, collapse="")))                     
+    names(seq) <- NULL
+    res <- list(nb=length(seq), nam=nam, seq=seq, com=NA)
+    class(res) = "alignment"
+    res
+}
+
+
+as.phyDat.MultipleAlignment <- function(x, ...){
+    if(inherits(x, "DNAMultipleAlignment"))
+        res <- phyDat.DNA(as.matrix(x))
+    if(inherits(x, "RNAMultipleAlignment"))
+        res <- phyDat.DNA(as.matrix(x))
+    if(inherits(x, "RNAMultipleAlignment"))
+        res <- phyDat.AA(as.matrix(x))
+    res
+}
+
+
+as.MultipleAlignment.phyDat <- function(x){
+    z = as.character(x)
+    nam = rownames(z)
+    type = attr(x, "type")
+    seq <- switch(type, 
+                  DNA = tolower(apply(z, 1, paste, collapse="")), 
+                  AA = toupper(apply(z, 1, paste, collapse="")))
+    if(type=="DNA") return(DNAMultipleAlignment(seq))
+    if(type=="AA") return(AAMultipleAlignment(seq))
+}
+
+
+phyDat2MultipleAlignment <- as.MultipleAlignment.phyDat
 
 
 as.phyDat.matrix <- function (x, ...) phyDat(data=x, ...)
@@ -509,7 +547,7 @@ as.DNAbin.phyDat <- function (x, ...)
  
 phyDat <- function (data, type="DNA", levels=NULL, return.index = TRUE,...) 
 {
-    if (class(data) == "DNAbin") type <- "DNA"
+    if (inherits(data,"DNAbin")) type <- "DNA"
     pt <- match.arg(type, c("DNA", "AA", "CODON", "USER"))  
     if(pt=="DNA") dat <- phyDat.DNA(data, return.index=return.index,...)
     if(pt=="AA") dat <- phyDat.AA(data, return.index=return.index, ...)
@@ -672,6 +710,10 @@ write.phyDat <- function(x, file, format="phylip",...){
 
 
 read.phyDat <- function(file, format="phylip", type="DNA", ...){
+    
+    formats <- c("phylip", "nexus", "interleaved", "sequential", "fasta", "clustal")
+    format <- match.arg(tolower(format), formats)
+    
     if(format=="nexus") data=read.nexus.data(file, ...)
     else {
         if(format=="phylip")format="interleaved"  #"sequential"
@@ -685,17 +727,18 @@ read.phyDat <- function(file, format="phylip", type="DNA", ...){
 }
 
 
-baseFreq <- function(obj, freq=FALSE, drop.unused.levels = FALSE){
-    if (class(obj) != "phyDat") 
+baseFreq <- function(obj, freq=FALSE, all=FALSE, drop.unused.levels = FALSE){
+    if (!inherits(obj,"phyDat")) 
         stop("data must be of class phyDat")
     labels <- attr(obj, "allLevels")
     weight <- attr(obj,"weight")
     n <- length(obj)    
     res <- numeric(length(labels))  
     D = diag(length(labels))   
-    for(i in 1:n)res <- res + colSums(D[obj[[i]],, drop=FALSE]*weight)      
-    if(!freq)res <- res/sum(res)
+    for(i in 1:n)res <- res + colSums(D[obj[[i]],, drop=FALSE]*weight)
     names(res) <- labels
+    if(!all) res <- res[attr(obj, "levels")]
+    if(!freq)res <- res/sum(res)
     if(drop.unused.levels) return(res[res>0])    
     res    
 }
@@ -764,15 +807,47 @@ subset.phyDat <- function (x, subset, select, site.pattern = TRUE,...)
 unique.phyDat <- function(x, incomparables=FALSE, ...) getCols(x, !duplicated(x))
 
 
+removeUndeterminedSites <- function(x, use.contrast=TRUE, undetermined=c("?", "n", "-"), ...){
+    nc <- attr(x, "nc")
+    nr <- attr(x, "nr")
+    contrast <- attr(x, "contrast")
+    if(use.contrast) ind <- which( (contrast %*% rep(1, nc)) == nc )
+    else ind <- sort(match(undetermined, attr(x, "allLevels")))
+    tmp <- x[[1]] %in% ind
+    for(i in 2:length(x)) tmp = tmp & (x[[i]] %in% ind)
+    if(any(tmp)) x <- getRows(x, (1:nr)[!tmp]) #getRows(x, -which(tmp))
+    x
+}
+
+
+removeParsUninfoSites <- function(data){
+    nr <- attr(data, "nr")
+    pis <- parsinfo(data)
+    if (length(pis) > 0) 
+        data <- getRows(data, c(1:nr)[-pis[, 1]], TRUE)
+}
+
+
+
 allSitePattern <- function(n,levels=c("a","c","g","t"), names=NULL){
     l=length(levels)
-    X=matrix(0, l^n,n)
+    X=vector("list", n)
+    if(is.null(names))names(X) = paste("t",1:n, sep="") 
+    else names(X)=names
     for(i in 1:n)
-    X[, i] = rep(rep(c(1:l), each=l^(i-1)),l^(n-i))
-    for(i in 1:l)X[X==i] = levels[i]
-    if(is.null(names))colnames(X) = paste("t",1:n, sep="")
-    else colnames(X)=names
-    phyDat.default(t(X), levels)
+        X[[i]] = rep(rep(levels, each=l^(i-1)),l^(n-i)) 
+    X = as.data.frame(X)
+    phyDat.default(X, levels, compress=FALSE) 
+} 
+
+
+constSitePattern <- function(n,levels=c("a","c","g","t"), names=NULL){
+    l=length(levels)
+    X=matrix(0, l,n)
+    X = matrix(rep(levels, each=n), n, l)
+    if(is.null(names))rownames(X) = paste("t",1:n, sep="")
+    else rownames(X)=names
+    phyDat.default(X, levels)
 } 
 
 
@@ -935,3 +1010,9 @@ read.aa <- function (file, format = "interleaved", skip = 0, nlines = 0,
     obj   
 }
 
+
+genlight2phyDat <- function(x, ambiguity=NA){
+    tmp <- as.matrix(x)
+    lev <- na.omit(unique(as.vector(tmp)))
+    phyDat(tmp, "USER", levels=lev, ambiguity=ambiguity)
+}
